"use strict";
/*
   Copyright (C) 2012 by Jeremy P. White <jwhite@codeweavers.com>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, see <http://www.gnu.org/licenses/>.
*/

/*--------------------------------------------------------------------------------------
**  SpiceWireReader
**      This class will receive messages from a WebSocket and relay it to a given
**  callback.  It will optionally save and pass along a header, useful in processing
**  the mini message format.
**--------------------------------------------------------------------------------------*/
function SpiceWireReader(sc, callback)
{
    this.sc = sc;
    this.callback = callback;
    this.needed = 0;
    this.blob_buffer = new myBlobBuilder();
    this.reader_lock = false;

    this.sc.ws.wire_reader = this;
    this.sc.ws.addEventListener('message', wire_blob_catcher);
}

SpiceWireReader.prototype =
{

    /*------------------------------------------------------------------------
    **  Process messages coming in from our WebSocket
    **      This function is complicated because we receive Blobs from
    **  our WebSocket, but we need ArrayBuffer's.  And getting ArrayBuffers
    **  is an asynchronous process (if done in the main loop).
    **----------------------------------------------------------------------*/
    inbound: function (blob)
    {

        /* Append the data to our ongoing buffer */
        if (blob)
        {
            DEBUG > 2 && console.log(this.sc.type + ": inbound: incoming size " + blob.size);
            this.blob_buffer.append(blob);
        }
        else
            DEBUG > 2 && console.log(this.sc.type + ": inbound:  Cleanup invocation.");

        var buffer =  this.blob_buffer.getBlob();
        if (this.needed > 0 && buffer.size >= this.needed && ! this.reader_lock)
        {
            DEBUG > 2 && console.log(this.sc.type + ": Have " + buffer.size + 
                        "; waiting for " + this.needed + "; proceeding.");

            /* Get the 'needed' bits into 'needed_blob'; leave the
               rest in the blob_buffer.
               FIXME:  this could likely be optimized, or use a cache
                       to avoid having to create a new BlobBuilder every time  */
            var needed_blob;
            var remainder_blob;
            if ('mozSlice' in buffer)
            {
                needed_blob = buffer.mozSlice(0, this.needed);
                remainder_blob = buffer.mozSlice(this.needed);
            }
            else
            {
                needed_blob = buffer.webkitSlice(0, this.needed);
                remainder_blob = buffer.webkitSlice(this.needed);
            }

            this.blob_buffer = new myBlobBuilder();
            this.blob_buffer.append(remainder_blob);


            /*----------------------------------------------------------------
            **  Read the needed blob into an ArrayBuffer
            **--------------------------------------------------------------*/
            var fr = new FileReader();
            fr.parent = this;
            fr.addEventListener('loadend', function(r) {
                if (r.target.result.byteLength > 0)
                {
                    DEBUG > 2 && hexdump_buffer(r.target.result);
                    this.parent.callback.call(this.parent.sc,
                        r.target.result, this.parent.saved_msg_header || undefined);
                }
                else
                {
                    /* FIXME: There is a bug in chrome, or a bug in this code
                    **        The 'Huh?' stuff below often triggers.
                    **        I filed http://code.google.com/p/chromium/issues/detail?id=125561 */
                    this.parent.sc.log_err(this.parent.type + ": Huh?  A zero byte result?");
                    console.log(r);
                    console.log(this);
                }
                this.parent.reader_lock = false;

                /*------------------------------------------------------------
                **  This is really important - this call keeps the initial
                **    receive event alive and processing. 
                **----------------------------------------------------------*/
                this.parent.inbound(null);
            });

            this.reader_lock = true;
            fr.readAsArrayBuffer(needed_blob);
        }
        else
        {
            /* MozBlobBuilder.getBlob() destroys the BlobBuilder, so we have to recreate it */
            this.blob_buffer = new myBlobBuilder();
            this.blob_buffer.append(buffer);
        }
    },

    request: function(n)
    {
        this.needed = n;
    },

    save_header: function(h)
    {
        this.saved_msg_header = h;
    },

    clear_header: function()
    {
        this.saved_msg_header = undefined;
    },
}

function wire_blob_catcher(e)
{
    DEBUG > 1 && console.log(">> WebSockets.onmessage");
    DEBUG > 1 && console.log("id " + this.wire_reader.sc.connection_id +"; type " + this.wire_reader.sc.type);
    SpiceWireReader.prototype.inbound.call(this.wire_reader, e.data);
}
